ArkInventory.LDB = {
	Bags = ArkInventory.Lib.LDB:NewDataObject( ArkInventory.Const.Program.Name .. "_" .. ArkInventory.Const.Slot.Data[ArkInventory.Const.Location.Bag].name, {
		type = "data source",
		icon = ArkInventory.Global.Location[ArkInventory.Const.Location.Bag].Texture,
		text = ArkInventory.Const.Slot.Data[ArkInventory.Const.Location.Bag].name,
	} ),
	Money = ArkInventory.Lib.LDB:NewDataObject( ArkInventory.Const.Program.Name .. "_Money", {
		type = "data source",
		icon = [[Interface\Icons\INV_Misc_Coin_02]],
		text = "Money",
	} ),
	Ammo = ArkInventory.Lib.LDB:NewDataObject( ArkInventory.Const.Program.Name .. "_Ammo", {
		type = "data source",
		icon = [[Interface\Icons\INV_Misc_Ammo_Bullet_02]],
		text = "Ammo",
	} ),
	Currency = ArkInventory.Lib.LDB:NewDataObject( ArkInventory.Const.Program.Name .. "_Currency", {
		type = "data source",
		icon = ArkInventory.Global.Location[ArkInventory.Const.Location.Token].Texture,
		text = "",
	} ),
	Pets = ArkInventory.Lib.LDB:NewDataObject( ArkInventory.Const.Program.Name .. "_Pet", {
		type = "data source",
		icon = ArkInventory.Global.Location[ArkInventory.Const.Location.Pet].Texture,
		text = "",
	} ),
	Mounts = ArkInventory.Lib.LDB:NewDataObject( ArkInventory.Const.Program.Name .. "_Mount", {
		type = "data source",
		icon = ArkInventory.Global.Location[ArkInventory.Const.Location.Mount].Texture,
		text = "",
	} ),
}

function ArkInventory.LDB.Bags.OnClick( frame, button )
	if button == "RightButton" then
		ArkInventory.MenuLDBBagsOpen( frame )
	else
		ArkInventory.Frame_Main_Toggle( ArkInventory.Const.Location.Bag )
	end
end

function ArkInventory.LDB.Bags:Update( )
	local loc_id = ArkInventory.Const.Location.Bag
	--ArkInventory.LDB.Bags.icon = ArkInventory.Global.Location[loc_id].Texture
	self.text = ArkInventory.Frame_Status_Update_Empty( loc_id, ArkInventory.Global.Me, true )
end



function ArkInventory.LDB.Money:Update( )
	ArkInventory.LDB.Money.text = ArkInventory.MoneyText( GetMoney( ) )
end

function ArkInventory.LDB.Money:OnEnter( )
	ArkInventory.MoneyFrame_OnEnter( self )	
end



function ArkInventory.LDB.Ammo.OnClick( frame, button )
	if button == "RightButton" then
		ArkInventory.MenuLDBAmmoOpen( frame )
	end
end

function ArkInventory.LDB.Ammo:Update( )
	
	local inv_id, texture = GetInventorySlotInfo( "RangedSlot" )
	local hw = GetInventoryItemLink( "player", inv_id )
	
	if not hw then
		
		-- no ranged weapon equipped
		self.icon = texture
		self.text = ""
		
	else
		
		local d1, d2 = GetInventoryItemDurability( inv_id )
		if d1 == d2 then
			d1 = ""
		elseif d1 == 0 then
			d1 = RED_FONT_COLOR_CODE .. ArkInventory.Localise["LDB_AMMO_WEAPON_BROKEN"] .. FONT_COLOR_CODE_CLOSE
		else
			if ArkInventory.db.char.option.ldb.ammo.durability then
				d1 = string.format( "%i%%", ( d1 / d2 ) * 100 )
			else
				d1 = ""
			end
		end
		
		local needsammo = false
		
		inv_id = GetInventorySlotInfo( "AmmoSlot" )
		
		local check = { 264, 5011, 266 }
		for _, v in pairs( check ) do
			local wt = GetSpellInfo( v )
			if IsEquippedItemType( wt ) then
				needsammo = true
				break
			end
		end
		
		if not needsammo then
			
			self.icon = GetItemIcon( hw )
			self.text = d1
			
		else
			
			local na = RED_FONT_COLOR_CODE .. ArkInventory.Localise["LDB_AMMO_NO_AMMO"] .. FONT_COLOR_CODE_CLOSE
			
			local ha = GetInventoryItemLink( "player", inv_id )
			
			if not ha then
				
				-- use weapon icon
				self.icon = GetItemIcon( hw )
				
				-- show durability
				if d1 == "" then
					self.text = na
				else
					self.text = string.format( "%s (%s)", na, d1 )
				end
				
			else
				
				-- use ammo icon
				self.icon = GetItemIcon( ha )
				
				-- get ammo count
				local c = GetItemCount( ha )
				if c == 0 then c = na end
				
				-- show count and durability
				if d1 == "" then
					self.text = string.format( "%s", c )
				else
					self.text = string.format( "%s (%s)", c, d1 )
				end
				
			end
			
		end
		
	end
	
end

function ArkInventory.LDB.Ammo:OnTooltipShow( )

	local inv_id = GetInventorySlotInfo( "RangedSlot" )
	local h = GetInventoryItemLink( "player", inv_id )
	
	if not h then
		self:AddLine( ArkInventory.Localise["LDB_AMMO_WEAPON_NONE"] )
	else
		self:AddLine( h )
	end

end



function ArkInventory.LDB.Currency:Update( )
	
	self.icon = ArkInventory.Global.Location[ArkInventory.Const.Location.Token].Texture
	self.text = ""
	
	-- expand all token headers
	local numTokenTypes = GetCurrencyListSize( )
	
	if numTokenTypes == 0 then return end
	
	for j = numTokenTypes, 1, -1 do
		local _, isHeader, isExpanded = GetCurrencyListInfo( j )
		if isHeader and not isExpanded then
			ExpandCurrencyList( j, 1 )
		end
	end
	
	local numTokenTypes = GetCurrencyListSize( )
	
	for j = 1, numTokenTypes do
	
		local name, isHeader, _, _, _, count, _, _, item = GetCurrencyListInfo( j )
	
		if not isHeader and name == ArkInventory.db.char.option.ldb.currency.track then
			self.icon = GetItemIcon( item )
			self.text = count
			return
		end
		
	end
	
end

function ArkInventory.LDB.Currency.OnClick( frame, button )
	if button == "RightButton" then
		ArkInventory.MenuLDBCurrencyOpen( frame )
	else
		ArkInventory.Frame_Main_Toggle( ArkInventory.Const.Location.Token )
	end
end

function ArkInventory.LDB.Currency:OnTooltipShow( )
	
	self:AddLine( CURRENCY )
	
	-- expand all token headers
	local numTokenTypes = GetCurrencyListSize( )
	
	if numTokenTypes == 0 then return end
	
	for j = numTokenTypes, 1, -1 do
		local _, isHeader, isExpanded = GetCurrencyListInfo( j )
		if isHeader and not isExpanded then
			ExpandCurrencyList( j, 1 )
		end
	end
	
	local numTokenTypes = GetCurrencyListSize( )
	
	for j = 1, numTokenTypes do
	
		local name, isHeader, _, _, _, count = GetCurrencyListInfo( j )
	
		if isHeader then
			self:AddLine( " " )
			self:AddLine( name )
		else
			if name == ArkInventory.db.char.option.ldb.currency.track then
				self:AddDoubleLine( name, count, 0, 1, 0, 0, 1, 0 )
			else
				self:AddDoubleLine( name, count, 1, 1, 1, 1, 1, 1 )
			end
		end
		
	end
	
end



function ArkInventory.LDB.Pets:Update( )
	
	if ArkInventory.db.char.option.ldb.pets.track then
		
		if ArkInventory.db.char.option.ldb.pets.track == 0 then
		
			self.icon = [[Interface\Icons\INV_Misc_Toy_03]]
			return
		
		else
			
			local type_id = "CRITTER"
			for k = 1, GetNumCompanions( type_id ) do
				local creatureID, creatureName, creatureSpellID, texture, active = GetCompanionInfo( type_id, k )
				if creatureSpellID == ArkInventory.db.char.option.ldb.pets.track then
					self.icon = texture
					return
				end
			end
			
			ArkInventory.db.char.option.ldb.pets.track = nil
			
		end
		
	end
	
	self.icon = ArkInventory.Global.Location[ArkInventory.Const.Location.Pet].Texture
	
end

function ArkInventory.LDB.Pets:OnTooltipShow( )
	
	local type_id = "CRITTER"
	local n = GetNumCompanions( type_id )
	
	if n == 0 then
		self:AddLine( "you have no pets" )
		return
	end
	
	if ArkInventory.db.char.option.ldb.pets.track then
		
		if ArkInventory.db.char.option.ldb.pets.track == 0 then
			
			self:AddLine( ArkInventory.Localise["GENERIC_RANDOM"] )
			return
			
		else
			
			for k = 1, GetNumCompanions( type_id ) do
				local creatureID, creatureName, creatureSpellID, texture, active = GetCompanionInfo( type_id, k )
				if creatureSpellID == ArkInventory.db.char.option.ldb.pets.track then
					self:AddLine( creatureName )
					return
				end
			end
			
			ArkInventory.db.char.option.ldb.pets.track = nil
			
		end
		
	end
	
	self:AddLine( "no pet selected" )

end

function ArkInventory.LDB.Pets.OnClick( frame, button )

	if button == "RightButton" then
		
		ArkInventory.MenuLDBPetsOpen( frame )
		
	else
		
		if ArkInventory.db.char.option.ldb.pets.track then
			
			local type_id = "CRITTER"
			local n = GetNumCompanions( type_id )
			
			if n == 0 then return end
			
			if ArkInventory.db.char.option.ldb.pets.track == 0 then
				
				local k = 1
				while true do
					k = random( 1, GetNumCompanions( type_id ) )
					local creatureID, creatureName, creatureSpellID, texture, active = GetCompanionInfo( type_id, k )
					if n == 1 or not active then break end
				end
				CallCompanion( type_id, k )
				return
				
			else
				
				for k = 1, n do
					local creatureID, creatureName, creatureSpellID, texture, active = GetCompanionInfo( type_id, k )
					if creatureSpellID == ArkInventory.db.char.option.ldb.pets.track then
						CallCompanion( type_id, k )
						return
					end
				end
				
				ArkInventory.OutputError( "you seem to have misplaced your pet" )
				
			end
			
		end
	
	end
	
end



function ArkInventory.LDB.Mounts:Update( )
	
	if ArkInventory.db.char.option.ldb.mounts.track then
		
		if ArkInventory.db.char.option.ldb.mounts.track == 0 then
		
			self.icon = ArkInventory.Global.Location[ArkInventory.Const.Location.Mount].Texture
			return
		
		else
			
			local type_id = "MOUNT"
			for k = 1, GetNumCompanions( type_id ) do
				local creatureID, creatureName, creatureSpellID, texture, active = GetCompanionInfo( type_id, k )
				if creatureSpellID == ArkInventory.db.char.option.ldb.mounts.track then
					self.icon = texture
					return
				end
			end
			
			ArkInventory.db.char.option.ldb.mounts.track = nil
			
		end
		
	end
	
	self.icon = ArkInventory.Global.Location[ArkInventory.Const.Location.Mount].Texture
	
end

function ArkInventory.LDB.Mounts:OnTooltipShow( )
	
	local type_id = "MOUNT"
	local n = GetNumCompanions( type_id )
	
	if n == 0 then
		self:AddLine( "you have no mounts" )
		return
	end
	
	if ArkInventory.db.char.option.ldb.mounts.track then
		
		if ArkInventory.db.char.option.ldb.mounts.track == 0 then
			
			self:AddLine( ArkInventory.Localise["GENERIC_RANDOM"] )
			return
			
		else
			
			for k = 1, GetNumCompanions( type_id ) do
				local creatureID, creatureName, creatureSpellID, texture, active = GetCompanionInfo( type_id, k )
				if creatureSpellID == ArkInventory.db.char.option.ldb.mounts.track then
					self:AddLine( creatureName )
					return
				end
			end
			
			ArkInventory.db.char.option.ldb.mounts.track = nil
			
		end
		
	end
	
	self:AddLine( "no mount selected" )

end

function ArkInventory.LDB.Mounts.OnClick( frame, button )

	if button == "RightButton" then
		
		ArkInventory.MenuLDBMountsOpen( frame )
		
	else
		
		if ArkInventory.db.char.option.ldb.mounts.track then
			
			local type_id = "MOUNT"
			local n = GetNumCompanions( type_id )
			
			if n == 0 then return end
			
			if ArkInventory.db.char.option.ldb.mounts.track == 0 then
				
				local k = 1
				while true do
					k = random( 1, GetNumCompanions( type_id ) )
					local creatureID, creatureName, creatureSpellID, texture, active = GetCompanionInfo( type_id, k )
					if n == 1 or not active then break end
				end
				CallCompanion( type_id, k )
				return
				
			else
				
				for k = 1, n do
					local creatureID, creatureName, creatureSpellID, texture, active = GetCompanionInfo( type_id, k )
					if creatureSpellID == ArkInventory.db.char.option.ldb.mounts.track then
						CallCompanion( type_id, k )
						return
					end
				end
				
				ArkInventory.OutputError( "you seem to have misplaced your mount" )
				
			end
			
		end
	
	end
	
end
